#Copyright 2009 Anton Vorobyov
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''
This script is used to compare two different database versions.
It shows removed/changed/new items with list of changed effects
and effects which were renamed
'''
import sys
sys.path.append("..")

from optparse import OptionParser
from dataFolder import dataFolder
import sqlite3
import os.path
import re

usage = "usage: %prog [--old=OLD] --new=NEW"
parser = OptionParser(usage=usage)
parser.add_option("-o", "--old", help="path to old sqlite database (if none specified script takes one currently used by pyfa)")
parser.add_option("-n", "--new", help="path to new sqlite database")
(options, args) = parser.parse_args()

effectsPath = os.path.join("..", "effects")
implemented = []

def getEffectStatus(effectName):
    if effectName in implemented: return 'y'
    else: return 'n'

for filename in os.listdir(effectsPath):
    basename, extension = filename.split('.')
    #Ignore non-py files and exclude 2 pyfa-specific 'effects'
    if extension == "py" and basename != "__init__" and basename != "customEffects":
        implemented.append(basename)

#Fetch database names from command line arguments
if options.old != None and options.new != None:
    oldDB = sqlite3.connect(options.old)
    newDB = sqlite3.connect(options.new)
elif options.new != None:
    import sql
    sql.connect("sqlite")
    oldDB = sql.getConnection()
    newDB = sqlite3.connect(options.new)
else:
    sys.stderr.write("You need to specify at least 1 database file. Run script with --help option for further info.\n")
    sys.exit()

print '[+] - new item\n[-] - removed item\n[*] - changed item\n  [+] - effect has \
been added to item\n  [-] - effect has been removed from item\n  [y] - effect is \
implemented\n  [n] - effect is not implemented\n\nItems:'

oldDict = {}
newDict = {}

#Compose list of item IDs and attach list of effect IDs to each
for database, dictionary in ((oldDB, oldDict), (newDB, newDict)):
    query = 'SELECT invTypes.typeID, dgmEffects.effectID FROM invTypes INNER JOIN \
    dgmTypeEffects ON dgmTypeEffects.typeID = invTypes.typeID INNER JOIN dgmEffects \
    ON dgmEffects.effectID = dgmTypeEffects.effectID WHERE invTypes.published = 1'
    c = database.cursor()
    c.execute(query)
    for row in c:
        if not row[0] in dictionary: dictionary[row[0]] = [row[1]]
        else:
            dictionary[row[0]].append(row[1])

#Lists contain IDs of items which have the same/changed set of effects' IDs
same = []
changed = []

#Iterates through item IDs from old database
for itemId in oldDict.iterkeys():
    if itemId in newDict:
        #If item with the same ID and exactly the same set of effects is found add it to 'same' list
        if oldDict[itemId] == newDict[itemId]:
            same.append(itemId)
        #Compares effects of old and new items, if both have the same effect it's deleted from the list on the fly
        else:
            i = 0
            rngI = len(oldDict[itemId])
            while i < rngI:
                tbd = False
                j = 0
                rngJ = len(newDict[itemId])
                while j < rngJ:
                    if oldDict[itemId][i] == newDict[itemId][j]:
                        tbd = True
                        del newDict[itemId][j]
                        rngJ -= 1
                        break
                    else:
                        j += 1
                if tbd == True:
                    del oldDict[itemId][i]
                    rngI -= 1
                else:
                    i += 1
            #If old and new item have no effects left - it means that items actually
            #had the same set of effects but it was sorted differently
            if not newDict[itemId] and not oldDict[itemId]:
                same.append(itemId)
            else:
                changed.append(itemId)

#Delete same items from both dictionaries
for itemId in same:
    del oldDict[itemId], newDict[itemId]

queryTypeName = 'SELECT invTypes.typeName FROM invTypes WHERE invTypes.typeID = ?'
queryEffectName = 'SELECT dgmEffects.effectName FROM dgmEffects WHERE dgmEffects.effectID = ?'

#Printjobs; effects' names are printed w/o &, tab symbols and spaces as pyfa also filters it
for itemId in changed:
    c = newDB.cursor()
    c.execute(queryTypeName, (itemId,))
    for row in c:
        print '\n[*] ' + row[0]
    for dict, db, sign in ((oldDict, oldDB, "-"), (newDict, newDB, "+")):
        for effect in dict[itemId]:
            c = db.cursor()
            c.execute(queryEffectName, (effect,))
            for row in c:
                print '  [%s|%s] %s' % (sign, getEffectStatus(re.sub("[&\t ]", "",row[0])), re.sub("[&\t ]", "",row[0]))

#Delete changed items; when same and changed items are deleted, we know that all items
#remaining in old dictionary were removed and all remaining in new dictionary were added
for itemId in changed:
    del oldDict[itemId], newDict[itemId]

for dict, db, sign in ((oldDict, oldDB, "-"), (newDict, newDB, "+")):
    for itemId in dict.iterkeys():
        c = db.cursor()
        c.execute(queryTypeName, (itemId,))
        for row in c:
            print '\n[%s] %s' % (sign, row[0])
        for effect in dict[itemId]:
            c = db.cursor()
            c.execute(queryEffectName, (effect,))
            for row in c:
                print '  [%s|%s] %s' % (sign, getEffectStatus(re.sub("[&\t ]", "",row[0])), re.sub("[&\t ]", "",row[0]))

#As pyfa uses effect name as unique ID, we have to keep track of changed effects' names
print '\nRenamed effects:'

oldDict = {}
newDict = {}

for database, dictionary in ((oldDB, oldDict), (newDB, newDict)):
    query = 'SELECT effectID, effectName FROM dgmEffects'
    c = database.cursor()
    c.execute(query)
    for row in c:
        dictionary[row[0]] = re.sub("[&\t ]", "",row[1])

for effectID in oldDict.iterkeys():
    if effectID in newDict:
        if oldDict[effectID] != newDict[effectID]:
            print '\n[' + getEffectStatus(oldDict[effectID]) + '] \"' + oldDict[effectID] + \
            '\"\n[' + getEffectStatus(newDict[effectID]) + '] \"' + newDict[effectID] + '\"'
